#
#
#
#
init_this = () ->
  init_google()
  #
  # init indexedDB
  indexedDB.deleteDatabase(dbName)
  request = indexedDB.open(dbName,2)

  request.onerror = (event) ->
    console.log("error on db request: " + request.errorCode + ".");

  request.onsuccess = (event) ->
    root.db = event.target.result;
    console.log("success on db request!");
    # set load call back after db has been initialized
    # first ajax call
    a1 = () ->
      $.ajax({
        type: "GET"
        url: "http://sels.tecnico.ulisboa.pt/datasources/report.csv"
        dataType: "text"
        success: (data) ->
          deal_with_data(data)
        error: (jqXHR, textStatus, errorThrown) ->
          write_error(textStatus)
      })
    # second ajax call
    a2 = () ->
      $.ajax({
        type: "GET"
        url: "http://sels.tecnico.ulisboa.pt/datasources/dictionary.csv"
        dataType: "text"
        success: (data) ->
          deal_with_dict(data)
        error: (jqXHR, textStatus, errorThrown) ->
          write_error(textStatus)
      });

    $.when(a1(),a2()).done (d1,d2) ->
      console.log "done (results.csv)"
      console.log "done (dictionary.csv)"
      # update organism and definition
      update_dict () ->
        console.log "finished updating description and organims columns in google tables"
        root.gdata.view.table = new google.visualization.DataView(root.gdata.data.table)
        setup_google_data()
        setup_google_vis()
        $('.show-with-data').show()

  request.onupgradeneeded = (event)->
    console.log "starting .onupgradeneeded..."
    root.db = event.target.result;
    # Create an objectStore for this database
    console.log "create object: dictionary"
    objectStore_dict = db.createObjectStore("dictionary", { keyPath: 'id' })
    console.log "create object: results"
    objectStore_resu = db.createObjectStore("results", { autoIncrement : true })
    # Create an index to search customers by name. We may have duplicates
    # so we can't use a unique index.
    console.log "create index: qseqid for results"
    objectStore_resu.createIndex("qseqid", "qseqid", { unique: false })

#
#
#
#
update_dict = (fun) ->
  transaction = db.transaction("dictionary","readonly");
  transaction.oncomplete = fun

  transaction.onerror = (event) ->
    console.log("error on transaction!! (with dictionary)");

  dictionaryObjectStore = transaction.objectStore("dictionary")

  qseqid_idx      = 6
  for el,idx in root.gdata.data.table.Lf
    row = el.c
    get_description dictionaryObjectStore, idx, row[qseqid_idx].v, (idx, value) ->
      description_idx = 4
      organism_idx    = 5
      root.gdata.data.table.setValue(idx, organism_idx, value.organism)
      root.gdata.data.table.setValue(idx, description_idx, value.description)


#
#
#
#
get_description = (dictionaryObjectStore,row_idx,qseqid,fun) ->

  req = dictionaryObjectStore.get("#{qseqid}")
  req.qseqid = qseqid
  req.row_idx = row_idx
  req.fun     = fun

  req.onsuccess = (event) ->
    req.fun(req.row_idx,event.target.result)

  req.onerror = (event) ->
    console.error "error retrieving qseqid from 'dictionary' table"




#
#
#
# Save data from results.csv to indexedDB
#
deal_with_data = ( input_data ) ->

  console.log("size of data for \"results\": #{input_data.length}");

  transaction = db.transaction("results", "readwrite");
  transaction.oncomplete = (event) ->
    #console.log("all done with transaction!")

  transaction.onerror = (event) ->
    console.log("error on transaction!! (with results)");

  resultsObjectStore = transaction.objectStore("results")

  csv2JSON input_data,2,"qseqid", (el) ->
    add_to_results(el,resultsObjectStore)

#
#
#
# Save data from dictionary.csv to indexedDB
#
deal_with_dict = ( input_data ) ->
  console.log("size of data for \"dictionary\": #{input_data.length}");

  transaction = db.transaction("dictionary", "readwrite");
  transaction.oncomplete = (event) ->
    #console.log("all done with transaction!")

  transaction.onerror = (event) ->
    console.log("error on transaction!! (with dictionary)");

  dictionaryObjectStore = transaction.objectStore("dictionary")

  csv2JSON input_data,1,undefined, (el) ->
    add_to_dictionary(el,dictionaryObjectStore)

#
#
#
#
add_to_dictionary = (el,obj_store)->
  obj_store.add(el)

#
#
# call back function to deal_with____
#
add_to_results = (el, obj_store) ->
  # get general data
  metadata = break_file(el.file)
  obj = [
    metadata.type
    metadata.remote_db
    metadata.query
    metadata.local_db
    ""
    ""
    el.qseqid
    el.sseqid
    el.evalue
    el.pident
    el.qcovs
  ]

  # update el not to store file without being parsed
  delete el.file
  el.type      = metadata.type
  el.remote_db = metadata.remote_db
  el.query     = metadata.query
  el.local_db  = metadata.local_db
  # push to table data
  obj_store.add(el)
  # add row to google DataTable
  add_to_google_table(obj)

add_to_google_table = (obj) ->
  root.gdata.data.table.addRow(obj)

#
#
#
write_error = (textStatus, msg="There was a problem when retrieving data, please try to reload the page.")->
  $('#error-msg').html("Error: (#{textStatus}) #{msg}")
